<?php /**
 * @file
 * Contains \Drupal\oauth_server_sso\Controller\DefaultController.
 */

namespace Drupal\oauth_server_sso\Controller;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormBuilder;
use Drupal\oauth_server_sso\MiniorangeOAuthServerConstants;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;
use Drupal\oauth_server_sso\DBQueries;
use Drupal\oauth_server_sso\handler;
use Drupal\Component\Utility\Html;
use Drupal\oauth_server_sso\Utilities;

class oauth_server_ssoController extends ControllerBase {

    protected $formBuilder;
    public function __construct(FormBuilder $formBuilder) {
        $this->formBuilder = $formBuilder;
    }

    public static function create(ContainerInterface $container) {
        return new static(
            $container->get("form_builder")
        );
    }

  public static function oauth_server_sso_authorize(){
    $config = \Drupal::config('oauth_server_sso.settings');
    if($config->get('enable_oauth_server') === false){
      Utilities::showErrorMessage('OAuth Server module is not enabled','Please navigate to the <b>Settings</b> tab and enable the <b>Master Switch</b>');
    }
    global $base_url;
    $request = \Drupal::request();
    $client_id = Html::escape($request->get('client_id') ?? '');
    handler::oauth_server_sso_validate_clientId($client_id);
    $redirect_uri = Html::escape($request->get('redirect_uri') ?? '');
    handler::oauth_server_sso_validate_redirectUrl($redirect_uri);
    if(\Drupal::currentUser()->isAuthenticated()){
      $user = \Drupal::currentUser();
      $allUserRolesInTheSystem = user_roles(TRUE);
      $currentUserIsAdmin = FALSE;
      foreach ($user->getRoles() as $roleName) {
       if($allUserRolesInTheSystem[$roleName]->isAdmin()){
         $currentUserIsAdmin = TRUE;
         break;
       }
      }
      if (!$currentUserIsAdmin )
        Utilities::showErrorMessage('Single Sign On not Allowed','This module is exclusively for use by Super Users/Administrators and is for demonstration purposes only.','The Single Sign On feature for the end users is available in the <a target="_blank" href="https://plugins.miniorange.com/drupal-oauth-server"> premium </a> version of the module.');

      $does_exist = DBQueries::get_user_id($user);
      if($does_exist == FALSE){
        DBQueries::insert_user_in_table($user);
      }
      $scope = Html::escape($request->get('scope') ?? '');
      $state =Html::escape($request->get('state') ?? '');
      handler::oauth_server_sso_validate_scope($scope,$user);
      $code = handler::generateRandom(25);
      DBQueries ::insert_code_from_user_id($code, $user);
      if(!empty(\Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_redirect_url'))){
        $url = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_redirect_url');
        if (strpos($url,'?') !== false) {
          $url =$url.'&code='.$code."&state=".$state;
        }else{
          $url =$url.'?code='.$code."&state=".$state;
        }
        \Drupal::logger('oauth_server_sso')->info('<pre><code>'.print_r('auth code sent '.$code, TRUE) .'</code></pre>');
        $code_time = time();
        DBQueries ::insert_code_expiry_from_user_id($code_time,$user);
        $response = new RedirectResponse($url);
        $response->send();
      }else{
        echo "Redirect URL not configured.";
      }
    }else{
     $rem_val = $_SERVER['QUERY_STRING'];
     $redirecting_url = $base_url.'/mo/oauth2/authorize?'.$rem_val;
     \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_red',$redirecting_url)->save();
     $response = new RedirectResponse($base_url.'/user/login?q=mo_redirect');
     $response->send();
    }
    return new Response();
  }

  public function oauth_server_sso_access_token(){
    $config = \Drupal::config('oauth_server_sso.settings');
    if($config->get('enable_oauth_server') === false){
      echo json_encode(array(
        "statusCode" => "ERROR",
        "statusMessage" => "OAuth Server module is not enabled",
      )); exit;
    }
    $request = \Drupal::request();
    $request_code = Html::escape($request->get('code') ?? '');
    $token_ep_error=json_encode(array("statusCode" => "ERROR","statusMessage" => "Invalid Grant type or Authorization Code",));
    if(empty($request_code)){
      \Drupal::logger('oauth_server_sso')->warning('Authorization code not received');
      echo $token_ep_error; exit;
    }
    \Drupal::logger('oauth_server_sso')->info('<pre><code>auth code received '.print_r($request_code, TRUE).'</code></pre>');
    $user_details = DBQueries ::get_code_from_user_id($request_code);
    if(!isset($user_details['user_id_val'])){
      \Drupal::logger('oauth_server_sso')->warning('Invalid authorization code received');
      echo $token_ep_error; exit;
    }
    $user =User::load($user_details['user_id_val']);
    if (!$user->hasRole('administrator')) {
       $add_logger=array(
        "statusCode" => 'ERROR',
        "statusMessage" => 'This module is exclusively for use by Super Users/Administrators and is for demonstration purposes only. The Premium version of the module offers the Single Sign-On feature for end users',
       );
       \Drupal::logger('oauth_server_sso')->warning('<pre>code>'.print_r($add_logger, TRUE).'</code></pre>');
        echo json_encode($add_logger); exit;
    }
    $code_request_time = $user_details['auth_code_expiry_time'];
    handler::ValidateAuthCode($code_request_time);
    $redirect_uri = Html::escape($request->get('redirect_uri') ?? '');
    handler::oauth_server_sso_validate_redirectUrl($redirect_uri);
    $client_id = Html::escape($request->get('client_id') ?? $request->server->get('PHP_AUTH_USER') ?? '');
    handler::oauth_server_sso_validate_clientId($client_id);
    $client_secret = Html::escape($request->get('client_secret') ?? $request->server->get('PHP_AUTH_PW') ?? '');
    handler::oauth_server_sso_validate_clientSecret($client_secret);
    $grant_type = Html::escape($request->get('grant_type') ?? '');
    handler::oauth_server_sso_validate_grant($grant_type);
    $access_token = handler::generateRandom(127);
    DBQueries ::insert_access_token_with_user_id($user_details['user_id_val'], $access_token);
    $server_response= array();
    $scope=$user_details['scope'];
    $scope_values = explode(" ", $scope);
    if(in_array('openid', $scope_values) && $config->get('enable_openid') === true){
      $id_token=self::generateIDToken($client_id,$user_details['user_id_val']);
      $server_response = array('id_token' => $id_token,'access_token' => $access_token, 'expires_in' => 3600, 'token_type' => 'Bearer', 'scope' => $scope);
    }else{
      $server_response = array('access_token' => $access_token, 'expires_in' => 3600, 'token_type' => 'Bearer', 'scope' => $scope);
    }
    $req_time = time();
    DBQueries::insert_access_token_expiry_time($user_details['user_id_val'],$req_time);
    \Drupal::logger('oauth_server_sso')->info('<pre><code>token sent '.print_r($server_response, TRUE).'</code></pre>');
    echo json_encode($server_response);exit;
    return new Response();
  }

  public function generateIDToken($client_id,$user_id){
        global $base_url;
        $user_obj = User::load($user_id);
        $uuid=  $user_obj->uuid();
        $email = $user_obj->getEmail();
        $name = $user_obj->getAccountName();
        $header_arr = [
            'alg' => 'HS256',
            'typ' => 'JWT'
        ];
        $payload_arr = array(
            'iss'        => $base_url,
            'sub'        => $uuid,
            'aud'        => $client_id,
            'iat'        => time(),
            'exp'        => time()+3600,
            'username'   => $name,
            'email'      => $email,
        );
        $base64UrlHeader = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode( json_encode( $header_arr ) ));
        $base64UrlPayload = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode( json_encode( $payload_arr ) ));
        $data = $base64UrlHeader.'.'.$base64UrlPayload;
        $client_secret = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_client_secret');
        //create signature
        $signature = hash_hmac('sha256', $data, $client_secret);
        $signature = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode($signature));
        $id_token = $data.'.'.$signature;
        return $id_token;
  }

  public function oauth_server_sso_user_info(){
    $config = \Drupal::config('oauth_server_sso.settings');
    if($config->get('enable_oauth_server') === false){
      echo json_encode(array(
        "statusCode" => "ERROR",
        "statusMessage" => "OAuth Server module is not enabled",
      )); exit;
    }
    $request = \Drupal::request();
    $header_value = $request->headers->get('Authorization');
    $access_token_received= (!is_null($header_value))&&(strpos($header_value, 'Bearer ') === 0)?substr($header_value, 7):$request->get('access_token');
    $user_info_error=json_encode(array("statusCode" => "invalid_token","statusMessage" => "Access token is expired or invalid",));
    if(empty($access_token_received)){
      \Drupal::logger('oauth_server_sso')->warning('Access token not received');
      echo $user_info_error; exit;
    }
    \Drupal::logger('oauth_server_sso')->info('<pre><code>access token received '.print_r($access_token_received, TRUE).'</code></pre>');
    $user_details= DBQueries::get_user_id_from_access_token($access_token_received);
    if(!isset($user_details['user_id_val'])){
      \Drupal::logger('oauth_server_sso')->warning('Invalid access token received');
      echo $user_info_error; exit;
    }
    $user =User::load($user_details['user_id_val']);
    if (!$user->hasRole('administrator')) {
      $add_logger=array(
       "statusCode" => 'ERROR',
       "statusMessage" => 'This module is exclusively for use by Super Users/Administrators and is for demonstration purposes only. The Premium version of the module offers the Single Sign-On feature for end users',
      );
      \Drupal::logger('oauth_server_sso')->warning('<pre>code>'.print_r($add_logger, TRUE).'</code></pre>' );
       echo json_encode($add_logger); exit;
    }
    $req_time =$user_details['access_token_request_time'];
    handler::ValidateAccessToken($req_time);
    $config = \Drupal::config('oauth_server_sso.settings');
    $scope=$user_details['scope'];
    $scope_values = explode(" ", $scope);
    $server_response = array();
    if(in_array('profile', $scope_values)) {
      $server_response= array(
        !empty($config->get('uid_attr_name'))?$config->get('uid_attr_name'):'id'=>$user->id(),
        !empty($config->get('uuid_attr_name'))?$config->get('uuid_attr_name'):'sub'=>$user->uuid(),
        !empty($config->get('name_attr_name'))?$config->get('name_attr_name'):'username'=>$user->getAccountName()
      );
    }
    if(in_array('email', $scope_values)) {
      $server_response = array_merge($server_response,array(
        !empty($config->get('mail_attr_name'))?$config->get('mail_attr_name'):'email'=>$user-> getEmail(),
      ));
    }
    $genericObject = (object)$server_response;
    \Drupal::logger('oauth_server_sso')->info('<pre><code>userinfo sent '.print_r($genericObject, TRUE).'</code></pre>');
    echo json_encode($genericObject);exit;
    return new Response();
  }

  public function miniorange_oauth_server_feedback_func(){
    global $base_url;
    $config = \Drupal::config('oauth_server_sso.settings');
     if (isset($_GET['miniorange_feedback_submit']) || isset($_GET['miniorange_feedback_skip'])){
      $modules_info = \Drupal::service('extension.list.module')->getExtensionInfo('oauth_server_sso');
      $modules_version = $modules_info['version'];
      $_SESSION['mo_other']="False";
      $user = \Drupal::currentUser();
      $email = isset($_GET['miniorange_feedback_email']) ? $_GET['miniorange_feedback_email'] : $config->get('oauth_server_sso_customer_admin_email');
      $email = empty($email) && !$user->isAnonymous() ? \Drupal\user\Entity\User::load($user->id())->getEmail() : $email;
      $client_name=$config->get('oauth_server_sso_client_name');
      $callback_url=$config->get('oauth_server_sso_redirect_url');
      if (isset($_GET['miniorange_feedback_skip'])){
        $message = $client_name != NULL
            ? 'Skipped feedback <br><br>Configured Client App: '.$client_name.'<br><br> Authorized Redirect/Callback URL: '.$callback_url
            : 'Skipped feedback <br><br>';

        $subject = 'Skipped Feedback for Drupal ' . \DRUPAL::VERSION . ' OAuth Server Module | '.$modules_version.' | PHP Version '.phpversion();
      }else{
        $reason = $_GET['deactivate_plugin'];
        $q_feedback = $_GET['query_feedback'];
        $message = $client_name != NULL
            ?'Reason: '.$reason.'<br>Feedback: '.$q_feedback . '<br><br>Selected App: '.$client_name.'<br><br>Authorize Endpoint: '.$callback_url
            :'Reason: '.$reason.'<br>Feedback: '.$q_feedback . '<br><br>';
      $subject = 'Drupal ' . \DRUPAL::VERSION . ' OAuth Server Module Feedback | '.$modules_version.' | PHP Version '.phpversion();
     }
      $customerKey= $config->get('oauth_server_sso_customer_id');
      $apikey = $config->get('oauth_server_sso_customer_api_key');
      if($customerKey==''){
        $customerKey="16555";
        $apikey="fFd2XcvTGDemZvbw1bcUesNJWEqKbbUq";
      }
      $currentTimeInMillis = self::get_oauth_timestamp();
      $stringToHash 		= $customerKey .  $currentTimeInMillis . $apikey;
      $hashValue 			= hash("sha512", $stringToHash);
      $customerKeyHeader 	= "Customer-Key: " . $customerKey;
      $timestampHeader 	= "Timestamp: " .  $currentTimeInMillis;
      $authorizationHeader= "Authorization: " . $hashValue;
      $fromEmail 			= $email;
      $query        = '[Drupal ' . \DRUPAL::VERSION . ' OAuth Server | '.$modules_version.' | PHP Version '.phpversion().' ]: ' . $message;
      $content='<div >Hello, <br><br>Company :<a href="'.$_SERVER['SERVER_NAME'].'" target="_blank" >'.$_SERVER['SERVER_NAME'].'</a><br><br>Email :<a href="mailto:'.$fromEmail.'" target="_blank">'.$fromEmail.'</a><br><br>Query :'.$query.'</div>';
      $fields = array(
        'customerKey'	=> $customerKey,
        'sendEmail' 	=> true,
        'email' 		=> array(
          'customerKey' 	=> $customerKey,
          'fromEmail' 	=> $fromEmail,
          'fromName' 		=> 'miniOrange',
          'toEmail' 		=>'drupalsupport@xecurify.com',
          'toName' 		=>'drupalsupport@xecurify.com',
          'subject' 		=> $subject,
          'content' 		=> $content
        ),
      );
      $field_string = json_encode($fields);
      $url = MiniorangeOAuthServerConstants::BASE_URL . '/moas/api/notify/send';
      $ch = curl_init($url);
      curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
      curl_setopt( $ch, CURLOPT_ENCODING, "" );
      curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
      curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
      curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );    # required for https urls
      curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
      curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', $customerKeyHeader,
        $timestampHeader, $authorizationHeader));
      curl_setopt( $ch, CURLOPT_POST, true);
      curl_setopt( $ch, CURLOPT_POSTFIELDS, $field_string);
      $content = curl_exec($ch);
      if(curl_errno($ch)){
        return json_encode(array("status"=>'ERROR','statusMessage'=>curl_error($ch)));
      }
      curl_close($ch);
    }

    \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->clear('miniorange_oauth_uninstall_status')->save();
    \Drupal::service('module_installer')->uninstall(['oauth_server_sso']);
    $uninstall_redirect = $base_url.'/admin/modules';
    \Drupal::messenger()->addMessage('The module has been successfully uninstalled.');
    return new RedirectResponse($uninstall_redirect);
  }

  /**
   * This function is used to get the timestamp value
   */
  public static function get_oauth_timestamp() {
    $url = MiniorangeOAuthServerConstants::BASE_URL .'/moas/rest/mobile/get-timestamp';
    $ch  = curl_init( $url );
    curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
    curl_setopt( $ch, CURLOPT_ENCODING, "" );
    curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
    curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
    curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
    curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, false ); // required for https urls
    curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
    curl_setopt( $ch, CURLOPT_POST, true );
    $content = curl_exec( $ch );
    if ( curl_errno( $ch ) ) {
      echo 'Error in sending curl Request';
      exit ();
    }
    curl_close( $ch );
    if(empty( $content )){
      $currentTimeInMillis = round( microtime( true ) * 1000 );
      $currentTimeInMillis = number_format( $currentTimeInMillis, 0, '', '' );
    }
    return empty( $content ) ? $currentTimeInMillis : $content;
  }

  public function openDemoRequestForm() {
    $response = new AjaxResponse();
    $modal_form = $this->formBuilder->getForm('\Drupal\oauth_server_sso\Form\MoOAuthRequestDemo');
    $response->addCommand(new OpenModalDialogCommand('Request 7-Days Full Feature Trial License', $modal_form, ['width' => '45%'] ) );
    return $response;
  }

  public function openSupportRequestForm() {
    $response = new AjaxResponse();
    $modal_form = $this->formBuilder->getForm('\Drupal\oauth_server_sso\Form\MoOAuthRequestSupport');
    $response->addCommand(new OpenModalDialogCommand('Support Request/Contact Us', $modal_form, ['width' => '40%'] ) );
    return $response;
  }

  public function openAddNewClientForm(){
    $response = new AjaxResponse();
    $provider_info['add_new_client_info'] = array(
      '#type' => 'item',
      '#markup' => $this->t('<p>You can configure only 1 application in free version of the module. Multiple applications are supported in <a href="licensing">PREMIUM</a> version of module</p>')
    );
    $ajax_form = new OpenModalDialogCommand('Add New OAuth/OpenID Client', $provider_info, ['width' => '40%']);
    $response->addCommand($ajax_form);
    return $response;
  }

  public function openAddMoreCallbackUrlsForm(){
    $response = new AjaxResponse();
    $provider_info['add_new_client_info'] = array(
      '#type' => 'item',
      '#markup' => $this->t('<p>You can add only 1 Callback/Redirect URL in free version of the module. Multiple Callback URLs are supported in <a href="licensing">PREMIUM</a> version of module</p>')
    );
    $ajax_form = new OpenModalDialogCommand('Add Callback URL', $provider_info, ['width' => '40%']);
    $response->addCommand($ajax_form);
    return $response;
  }

  public function openRemoveAccountForm() {
    $response = new AjaxResponse();
    $modal_form = $this->formBuilder->getForm('\Drupal\oauth_server_sso\Form\MiniorangeOAuthRemoveAccount');
    $response->addCommand(new OpenModalDialogCommand('Remove Account', $modal_form, ['width' => '800']));
    return $response;
  }
}
