<?php

/**
 * @file
 * Contains \Drupal\oauth_server_sso\Form\MiniorangeOAuthServerCustomerSetup.
 */
 namespace Drupal\oauth_server_sso\Form;
 use Drupal\Core\Form\FormStateInterface;
 use Drupal\oauth_server_sso\MiniorangeOAuthServerCustomer;
 use Drupal\Core\Form\FormBase;
 use Drupal\oauth_server_sso\Utilities;
 use Drupal\Core\Url;

class MiniorangeOAuthServerCustomerSetup extends FormBase
{
    public function getFormId()
    {
        return 'oauth_server_sso_customer_setup';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        global $base_url;
        $current_status = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_status');

        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "oauth_server_sso/oauth_server_sso.style_settings",
                )
            ),
        );

        if ($current_status == 'VALIDATE_OTP') {
            $form['header_top_style_1'] = array('#markup' => '<div class="mo_oauth_table_layout_1">');

            $form['markup_top'] = array(
                '#markup' => '<div class="mo_oauth_table_layout mo_oauth_container_login">',
            );

            $admin_mail = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_email');

            $form['markup_top_vt_start'] = array(
                '#type' => 'fieldset',
                '#title' => t('OTP Validation'),
                '#attributes' => array('style' => 'padding:2% 2% 5%; margin-bottom:2%'),
                '#markup' => '<br><hr><br>',
            );

            $form['markup_top_vt_start']['mo_user_profile'] = array(
                '#markup' => '<h3>Please enter the OTP sent to <i>' . $admin_mail . '</i>: </h3><br>'
            );

            $form['markup_top_vt_start']['oauth_server_sso_customer_otp_token'] = array(
                '#type' => 'textfield',
                '#title' => t('OTP'),
                '#attributes' => array('style' => 'width:30%;', 'placeholder' => 'Enter OTP'),
            );

            $form['markup_top_vt_start']['mo_btn_brk'] = array('#markup' => '<br><br>');

            $form['markup_top_vt_start']['oauth_server_sso_customer_validate_otp_button'] = array(
                '#type' => 'submit',
                '#value' => t('Validate OTP'),
                '#submit' => array('::oauth_server_sso_validate_otp_submit'),
            );

            $form['markup_top_vt_start']['oauth_server_sso_customer_setup_resendotp'] = array(
                '#type' => 'submit',
                '#value' => t('Resend OTP'),
                '#submit' => array('::oauth_server_sso_resend_otp'),
            );

            $form['markup_top_vt_start']['oauth_server_sso_customer_setup_back'] = array(
                '#type' => 'submit',
                '#value' => t('Back'),
                '#submit' => array('::oauth_server_sso_back'),
                '#suffix' => '<br><br><br><br><br><br><br><br>',
            );

            Utilities::nofeaturelisted($form, $form_state);
            $form['markup_top_vt_start']['header_top_div_end'] = array('#markup' => '</div>');
            Utilities::moOAuthServerShowCustomerSupportIcon($form, $form_state);

            return $form;

        } elseif ($current_status == 'PLUGIN_CONFIGURATION') {
            $modules_info = \Drupal::service('extension.list.module')->getExtensionInfo('oauth_server_sso');
            $modules_version = $modules_info['version'];
            $form['markup_top'] = array(
                '#markup' => t('<div class="mo_oauth_table_layout_1"><div class="mo_oauth_table_layout mo_oauth_container_login">
                                  <div class="mo_oauth_server_welcome_message">Thank you for registering with miniOrange</div>')
            );

            $form['mo_oauth_profile_details_tab'] = array(
                '#type' => 'fieldset',
                '#title' => t('Profile Details:'),
                '#attributes' => array( 'style' => 'padding:2% 2% 6%; margin-bottom:2%' ),
            );

            $header = array( t('Attribute'),  t('Value'),);
            $options = array(
                array( 'Customer Email', \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_email'),),
                array( 'Customer ID', \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_id'),),
                array( 'Drupal Version', \DRUPAL::VERSION,),
                array( 'PHP Version', phpversion(),),
                array( 'Module Version', $modules_version,),
            );

            $form['mo_oauth_profile_details_tab']['fieldset']['customerinfo'] = array(
                '#theme' => 'table',
                '#header' => $header,
                '#rows' => $options,
                '#prefix' => '<br>',
                '#attributes' => array( 'style' => 'margin:1% 0% 7%;' ),
            );

            $form['mo_oauth_profile_details_tab']['miniorange_oauth_customer_Remove_Account_info'] = array(
                '#markup' => t('<br/><h4>Remove Account:</h4><p>This section will help you to remove your current
                        logged in account without losing your current configurations.</p>')
            );

            $form['mo_oauth_profile_details_tab']['miniorange_oauth_customer_Remove_Account'] = array(
                '#type' => 'link',
                '#title' => $this->t('Remove Account'),
                '#url' => Url::fromRoute('oauth_server_sso.removeAccount'),
                '#attributes' => [
                    'class' => [
                        'use-ajax',
                        'button',
                    ],
                ],
                '#suffix' => '</div>'
            );

            $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
            Utilities::nofeaturelisted($form, $form_state);
            $form['mo_markup_div_end2'] = array('#markup' => '</div>');
            Utilities::moOAuthServerShowCustomerSupportIcon($form, $form_state);
            return $form;

        }else if($current_status == 'already_registered'){
            $form['header_top_style_1'] = array('#markup' => '<div class="mo_oauth_table_layout_1">');

            $form['markup_top'] = array(
                '#markup' => '<div class="mo_oauth_table_layout mo_oauth_container_login login_form"><h2>Login with mini<span class="orange">O</span><span>range</h2><hr>',
            );

            $form['mo_oauth_client_customer_email'] = array(
                '#type' => 'textfield',
                '#title' => t('Email'),
                '#required' => True,
                '#attributes' => array(
                    'style' => 'width:50%'
                ),
            );

            $form['mo_oauth_client_customer_password'] = array(
                '#type' => 'password',
                '#title' => t('Password'),
                '#required' => True,
                '#attributes' => array(
                    'style' => 'width:50%'
                ),
            );

            $form['login_submit'] = array(
                '#type' => 'submit',
                '#button_type' => 'primary',
                '#value' => t('Login')
            );

            $form['back_button'] = array(
                '#type' => 'submit',
                '#submit' => array('::back_to_register_tab'),
                '#value' => t('Create an account?'),
                '#suffix' => '</div>'
            );

            Utilities::nofeaturelisted($form, $form_state);
            $form['mo_markup_div_end2'] = array('#markup' => '</div>');
            Utilities::moOAuthServerShowCustomerSupportIcon($form, $form_state);

            return $form;
        }
        $form['header_top_style_1'] = array('#markup' => '<div class="mo_oauth_table_layout_1">');

        $form['markup_top'] = array(
            '#markup' => '<div class="mo_oauth_table_layout mo_oauth_container_login login_form"><h2>Register with mini<span class="orange">O</span><span>range</h2><hr>',
        );

        $form['mo_oauth_registration_tab'] = array(
            '#type' => 'fieldset',
            '#title' => t('Why should I register?'),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

        $form['mo_oauth_registration_tab']['markup_msg_1'] = array(
            '#markup' => t('<br><div class="mo_oauth_highlight_background_note">You should register so that in case you need help, we can help you with step-by-step instructions.
                <b>You will also need a miniOrange account to upgrade to the premium version of the module.</b>
                We do not store any information except the email that you will use to register with us. Please enter a valid email ID that you have access to. We will send OTP to this email for verification.</div><br>')
        );
        $form['mo_oauth_registration_tab']['mo_register'] = array(
                '#markup' => t('<div class="mo_oauth_highlight_background_note" style="width: auto">If you face any issues during registration then you can <b><a href="https://www.miniorange.com/businessfreetrial" target="_blank">click here</a></b> to register and use the same credentials below to login into the module.</div><br><div id="Register_Section">'),
        );

        $form['mo_oauth_registration_tab']['oauth_server_sso_customer_setup_username'] = array(
            '#type' => 'email',
            '#title' => t('Email'),
            '#attributes' => array('style' => 'width:54%;', 'placeholder' => 'Enter your email'),
            '#required' => TRUE,
        );

        $form['mo_oauth_registration_tab']['oauth_server_sso_customer_setup_password'] = array(
            '#type' => 'password_confirm',
            '#required' => TRUE,
        );

        $form['mo_oauth_registration_tab']['oauth_server_sso_customer_setup_button'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => t('Register'),
            '#attributes' => ['style' => 'float:left;'],
            '#prefix' => '<br><span>',
        );

        $form['mo_oauth_registration_tab']['oauth_server_sso_customer_setup_alredy_registered_button'] = array(
            '#type' => 'submit',
            '#value' => t('Already have an account?'),
            '#submit' => array('::already_registred'),
            '#limit_validation_errors' => array(),
            '#suffix' => '</span>',
        );

        $form['mo_oauth_registration_tab']['markup_divEnd'] = array(
            '#markup' => '</div></div>'
        );

        Utilities::nofeaturelisted($form, $form_state);
        $form['mo_markup_div_end2'] = array('#markup' => '</div>');
        Utilities::moOAuthServerShowCustomerSupportIcon($form, $form_state);

        return $form;
    }

    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        $current_status = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_status');
        if($current_status == 'already_registered'){
            $username = trim($form['mo_oauth_client_customer_email']['#value']);
            $phone = '';
            $password = trim($form['mo_oauth_client_customer_password']['#value']);
        }else{
            $username = trim($form['mo_oauth_registration_tab']['oauth_server_sso_customer_setup_username']['#value']);
            $phone='';
            $password = trim($form['mo_oauth_registration_tab']['oauth_server_sso_customer_setup_password']['#value']['pass1']);
        }

        if (empty($username) || empty($password)) {
            \Drupal::messenger()->addMessage(t('The <b><u>Email </u></b> and <b><u>Password</u></b> fields are mandatory.'), 'error');
            return;
        }
        if (!\Drupal::service('email.validator')->isValid($username)) {
            \Drupal::messenger()->addMessage(t('The email address <i>' . $username . '</i> is not valid.'), 'error');
            return;
        }
        $customer_config = new MiniorangeOAuthServerCustomer($username, $phone, $password, NULL);
        $check_customer_response = json_decode($customer_config->checkCustomer());

        if (is_object($check_customer_response) && $check_customer_response->status == 'CUSTOMER_NOT_FOUND') {
            $config = \Drupal::configFactory()->getEditable('oauth_server_sso.settings');
            $config->set('oauth_server_sso_customer_admin_email', $username)
                   ->set('oauth_server_sso_customer_admin_phone', $phone)
                   ->set('oauth_server_sso_customer_admin_password', $password)
                    ->save();
            if($current_status == 'already_registered'){
                \Drupal::messenger()->addMessage(t('Account with username @username is not registered with miniOrange, Please Register with miniOrange to login', [
                    '@username' => $username
                ]),'error');
                return;
            }
            $send_otp_response = json_decode($customer_config->sendOtp());

            if (is_object($send_otp_response) && $send_otp_response->status == 'SUCCESS') {
                \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_tx_id', $send_otp_response->txId)->save();
                $current_status = 'VALIDATE_OTP';
                \Drupal::messenger()->addMessage(t('Verify email address by entering the passcode sent to @username', [
                    '@username' => $username
                ]));
                \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_status', $current_status)->save();
            }elseif (is_object($send_otp_response) && $send_otp_response->status == 'FAILED'){
                \Drupal::messenger()->addMessage(t('An error has been occured. Please try after some time or contact us at <a href="mailto:drupalsupport@xecurify.com" target="_blank">drupalsupport@xecurify.com</a>.'), 'error');
            }
        } elseif (is_object($check_customer_response) && $check_customer_response->status == 'CURL_ERROR') {
            \Drupal::messenger()->addMessage(t('cURL is not enabled. Please enable cURL'), 'error');
        } else {
          $content = $customer_config->getCustomerKeys();
          if(isset($content)){
            $customer_keys_response = json_decode($content);
          }else{
            $customer_keys_response = null;
          }

            if (json_last_error() == JSON_ERROR_NONE && isset($customer_keys_response)) {
                $config = \Drupal::configFactory()->getEditable('oauth_server_sso.settings');
                $config->set('oauth_server_sso_customer_id', $customer_keys_response->id)
                        ->set('oauth_server_sso_customer_admin_token', $customer_keys_response->token)
                        ->set('oauth_server_sso_customer_admin_email', $username)
                        ->set('oauth_server_sso_customer_admin_phone', $phone)
                        ->set('oauth_server_sso_customer_api_key', $customer_keys_response->apiKey)
                        ->set('oauth_server_sso_status', 'PLUGIN_CONFIGURATION')
                        ->save();
                \Drupal::messenger()->addMessage(t('Successfully retrieved your account.'));
                $register_to_upgrade = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_redi_upgrade');
                if ($register_to_upgrade == 1) {
                    \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_redi_upgrade', '0')->save();
                    global $base_url;
                    $response = new RedirectResponse($base_url . "/admin/config/people/oauth_server_sso/licensing");
                    $response->send();
                }
            } else if (is_object($check_customer_response) && $check_customer_response->status == 'TRANSACTION_LIMIT_EXCEEDED') {
                \Drupal::messenger()->addMessage(t('An error has occurred. Please try after some time or contact us at <a href="mailto:drupalsupport@xecurify.com" target="_blank">drupalsupport@xecurify.com</a>.'), 'error');
            } else {
                \Drupal::messenger()->addMessage(t('Invalid credentials'), 'error');
            }
        }
    }

    public function oauth_server_sso_back(&$form, $form_state)
    {
        $config = \Drupal::configFactory()->getEditable('oauth_server_sso.settings');
       $config->set('oauth_server_sso_status', 'CUSTOMER_SETUP')
               ->clear('miniorange_oauth_server_sso_customer_admin_email')
                ->clear('oauth_server_sso_customer_admin_phone')
                ->clear('oauth_server_sso_tx_id')
                ->save();
        \Drupal::messenger()->addMessage(t('Register/Login with your miniOrange Account'), 'status');
    }

    public static function setup_call(array &$form, FormStateInterface $form_state){
        Utilities::schedule_a_call($form, $form_state);
    }

    public function oauth_server_sso_resend_otp(&$form, $form_state)
    {
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->clear('oauth_server_sso_tx_id')->save();
        $username = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_email');
        $phone = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_phone');
        $customer_config = new MiniorangeOAuthServerCustomer($username, $phone, NULL, NULL);
        $send_otp_response = json_decode($customer_config->sendOtp());
        if (is_object($send_otp_response) && $send_otp_response->status == 'SUCCESS') {
            // Store txID.
            \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_tx_id', $send_otp_response->txId)->save();
            $current_status = 'VALIDATE_OTP';
            \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_status', $current_status)->save();
            \Drupal::messenger()->addMessage(t('Verify email address by entering the passcode resent to @username', array('@username' => $username)));
        }
    }

    public function oauth_server_sso_validate_otp_submit(&$form, $form_state)
    {
        $otp_token = trim($form['markup_top_vt_start']['oauth_server_sso_customer_otp_token']['#value']);
        if ($otp_token == NULL) {
            \Drupal::messenger()->addMessage(t('Please enter OTP first.'), 'error');
            return;
        }
        $username = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_email');
        $phone = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_phone');
        $tx_id = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_tx_id');
        $customer_config = new MiniorangeOAuthServerCustomer($username, $phone, NULL, $otp_token);
        $validate_otp_response = json_decode($customer_config->validateOtp($tx_id));
        if (is_object($validate_otp_response) && $validate_otp_response->status == 'SUCCESS') {
            \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->clear('oauth_server_sso_tx_id')->save();
            $password = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_password');
            $customer_config = new MiniorangeOAuthServerCustomer($username, $phone, $password, NULL);
            $create_customer_response = json_decode($customer_config->createCustomer());
            if (is_object($create_customer_response) && $create_customer_response->status == 'SUCCESS') {
                $config = \Drupal::configFactory()->getEditable('oauth_server_sso.settings');
                $config->set('oauth_server_sso_status', 'PLUGIN_CONFIGURATION')
                ->set('oauth_server_sso_customer_admin_email', $username)
                ->set('oauth_server_sso_customer_admin_phone', $phone)
                ->set('oauth_server_sso_customer_admin_token', $create_customer_response->token)
                ->set('oauth_server_sso_customer_id', $create_customer_response->id)
                ->set('oauth_server_sso_customer_api_key', $create_customer_response->apiKey)
                ->save();
                \Drupal::messenger()->addMessage(t('Account created successfully.'));
            } else if (is_object($create_customer_response) && trim($create_customer_response->message) == 'Email is not enterprise email.' || ($create_customer_response->status) == "INVALID_EMAIL_QUICK_EMAIL") {
                \Drupal::messenger()->addMessage(t('There was an error creating an account for you. You may have entered an invalid Email-Id
                        <strong>(We discourage the use of disposable emails) </strong>
                        <br>Please try again with a valid email.'), 'error');
                \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_status', 'CUSTOMER_SETUP')->save();
            } else {
                \Drupal::messenger()->addMessage(t('Error in creating an account for you. Please try again.'), 'error');
            }
        } else {
            \Drupal::messenger()->addMessage(t('Invalid OTP provided. Please enter the correct OTP'), 'error');
        }
    }

    public function already_registred(){
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_status', 'already_registered')->save();
    }

    public function back_to_register_tab(){
        \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('oauth_server_sso_status', '')->save();
    }
}