<?php
/**
 * @file
 * Contains miniOrange Customer class.
 */

/**
 * @file
 * This class represents configuration for customer.
 */
namespace Drupal\oauth_server_sso;
use Drupal\oauth_server_sso\Utilities;
use GuzzleHttp\Exception\RequestException;

class MiniorangeOAuthServerCustomer {

    public $email;
    public $phone;
    public $customerKey;
    public $transactionId;
    public $password;
    public $otpToken;
    private $defaultCustomerId;
    private $defaultCustomerApiKey;

    /**
     * Constructor.
     */
    public function __construct($email, $phone, $password, $otp_token) {
        $this->email = $email;
        $this->phone = $phone;
        $this->password = $password;
        $this->otpToken = $otp_token;
        $this->defaultCustomerId = "16555";
        $this->defaultCustomerApiKey = "fFd2XcvTGDemZvbw1bcUesNJWEqKbbUq";
    }

    /**
     * Check if customer exists.
     */
    public function checkCustomer() {
        if (!Utilities::isCurlInstalled()) {
            return json_encode(array(
                "status" => 'CURL_ERROR',
                "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
            ));
        }

        $url = MiniorangeOAuthServerConstants::BASE_URL . '/moas/rest/customer/check-if-exists';
        $ch = curl_init($url);
        $email = $this->email;

        $fields = array(
            'email' => $email,
        );
        $field_string = json_encode($fields);

        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
        curl_setopt($ch, CURLOPT_ENCODING, "");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

        curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json', 'charset: UTF - 8',
            'Authorization: Basic',
        ));
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $field_string);
        $content = curl_exec($ch);
        if (curl_errno($ch)) {
            $error = array(
                '%method' => 'checkCustomer',
                '%file' => 'customer_setup.php',
                '%error' => curl_error($ch),
            );
            \Drupal::logger('oauth_server_sso')->error('Error at %method of %file: %error <pre>code>' . print_r($error, TRUE) . '</code></pre>' );
        }
        curl_close($ch);

        return $content;
    }

    /**
     * Create Customer.
     */
    public function createCustomer() {
        if (!Utilities::isCurlInstalled()) {
            return json_encode(array(
                "statusCode" => 'ERROR',
                "statusMessage" => '. Please check your configuration.',
            ));
        }
        $url = MiniorangeOAuthServerConstants::BASE_URL . '/moas/rest/customer/add';
        $ch = curl_init($url);
        $fields = array(
            'companyName' => $_SERVER['SERVER_NAME'],
            'areaOfInterest' => 'DRUPAL 8 OAuth Server',
            'email' => $this->email,
            'phone' => $this->phone,
            'password' => $this->password,
        );
        $field_string = json_encode($fields);

        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
        curl_setopt($ch, CURLOPT_ENCODING, "");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

        curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json',
            'charset: UTF - 8',
            'Authorization: Basic',
        ));
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $field_string);
        $content = curl_exec($ch);

        if (curl_errno($ch)) {
            $error = array(
                '%method' => 'createCustomer',
                '%file' => 'customer_setup.php',
                '%error' => curl_error($ch),
            );
            \Drupal::logger('oauth_server_sso')->error('Error at %method of %file: %error <pre>code>' . print_r($error, TRUE) . '</code></pre>' );
        }
        curl_close($ch);
        return $content;
    }

    /**
     * Get Customer Keys.
     */
    public function getCustomerKeys() {
        if (!Utilities::isCurlInstalled()) {
            return json_encode(array(
                "apiKey" => 'CURL_ERROR',
                "token" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
            ));
        }

        $url = MiniorangeOAuthServerConstants::BASE_URL . '/moas/rest/customer/key';
        $ch = curl_init($url);
        $email = $this->email;
        $password = $this->password;

        $fields = array(
            'email' => $email,
            'password' => $password,
        );
        $field_string = json_encode($fields);

        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
        curl_setopt($ch, CURLOPT_ENCODING, "");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

        curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json',
            'charset: UTF - 8',
            'Authorization: Basic',
        ));
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $field_string);
        $content = curl_exec($ch);
        if (curl_errno($ch)) {
            $error = array(
                '%method' => 'getCustomerKeys',
                '%file' => 'customer_setup.php',
                '%error' => curl_error($ch),
            );
            \Drupal::logger('oauth_server_sso')->error('Error at %method of %file: %error <pre>code>' . print_r($error, TRUE) . '</code></pre>' );
        }
        curl_close($ch);

        return $content;
    }

    /**
     * Send OTP.
     */
    public function sendOtp() {
        if (!Utilities::isCurlInstalled()) {
            return json_encode(array(
                "status" => 'CURL_ERROR',
                "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
            ));
        }
        $url = MiniorangeOAuthServerConstants::BASE_URL . '/moas/api/auth/challenge';
        $ch = curl_init($url);
        $customer_key = $this->defaultCustomerId;
        $api_key = $this->defaultCustomerApiKey;

        $username = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_customer_admin_email');

        /* Current time in milliseconds since midnight, January 1, 1970 UTC. */
        $current_time_in_millis = round(microtime(TRUE) * 1000);

        /* Creating the Hash using SHA-512 algorithm */
        $string_to_hash = $customer_key . $current_time_in_millis . $api_key;
        $hash_value = hash("sha512", $string_to_hash);

        $customer_key_header = "Customer-Key: " . $customer_key;
        $timestamp_header = "Timestamp: " . $current_time_in_millis;
        $authorization_header = "Authorization: " . $hash_value;

        $fields = array(
            'customerKey' => $customer_key,
            'email' => $username,
            'authType' => 'EMAIL',
        );
        $field_string = json_encode($fields);

        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
        curl_setopt($ch, CURLOPT_ENCODING, "");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

        curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", $customer_key_header,
            $timestamp_header, $authorization_header,
        ));
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $field_string);
        $content = curl_exec($ch);

        if (curl_errno($ch)) {
            $error = array(
                '%method' => 'sendOtp',
                '%file' => 'customer_setup.php',
                '%error' => curl_error($ch),
            );
            \Drupal::logger('oauth_server_sso')->error('Error at %method of %file: %error <pre>code>' . print_r($error, TRUE) . '</code></pre>' );
        }
        curl_close($ch);
        return $content;
    }

    /**
     * Validate OTP.
     */
    public function validateOtp($transaction_id) {
        if (!Utilities::isCurlInstalled()) {
            return json_encode(array(
                "status" => 'CURL_ERROR',
                "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
            ));
        }

        $url = MiniorangeOAuthServerConstants::BASE_URL . '/moas/api/auth/validate';
        $ch = curl_init($url);

        $customer_key = $this->defaultCustomerId;
        $api_key = $this->defaultCustomerApiKey;

        $current_time_in_millis = round(microtime(TRUE) * 1000);

        $string_to_hash = $customer_key . $current_time_in_millis . $api_key;
        $hash_value = hash("sha512", $string_to_hash);

        $customer_key_header = "Customer-Key: " . $customer_key;
        $timestamp_header = "Timestamp: " . $current_time_in_millis;
        $authorization_header = "Authorization: " . $hash_value;

        $fields = array(
            'txId' => $transaction_id,
            'token' => $this->otpToken,
        );

        $field_string = json_encode($fields);

        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
        curl_setopt($ch, CURLOPT_ENCODING, "");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

        curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", $customer_key_header,
            $timestamp_header, $authorization_header,
        ));
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $field_string);
        $content = curl_exec($ch);

        if (curl_errno($ch)) {
            $error = array(
                '%method' => 'validateOtp',
                '%file' => 'customer_setup.php',
                '%error' => curl_error($ch),
            );
            \Drupal::logger('oauth_server_sso')->error('Error at %method of %file: %error <pre>code>' . print_r($error, TRUE) . '</code></pre>' );
        }
        curl_close($ch);
        return $content;
    }

    function callService($url, $fields, $addExtendedHeader = FALSE, $logError = TRUE){
        if (!Utilities::isCurlInstalled()) {
            return json_encode(array(
                "statusCode" => 'ERROR',
                "statusMessage" => 'cURL is not enabled on your site. Please enable the cURL module.',
            ));
        }
        $fieldString = is_string($fields) ? $fields : json_encode($fields);
        $header = $this->getHeader($addExtendedHeader);
        try{
            $response = \Drupal::httpClient()
                ->post($url, [
                    'body' => $fieldString,
                    'allow_redirects' => TRUE,
                    'http_errors' => FALSE,
                    'decode_content'  => true,
                    'verify' => FALSE,
                    'headers' =>$header
                ]);
            return $response->getBody()->getContents();
        }
        catch (RequestException $exception)
        {
            if($logError){
                $error = array(
                    '%apiName' => explode("moas",$url)[1],
                    '%error' => $exception->getResponse()->getBody()->getContents(),
                );
                \Drupal::logger('oauth_server_sso')->error('Error at %method of %file: %error <pre>code>' . print_r($error, TRUE) . '</code></pre>' );
            }
        }
    }

    function getHeader($addExtendedHeader = FALSE){

        $header = array(
            'Content-Type'=>'application/json', 'charset'=>'UTF - 8',
            'Authorization'=> 'Basic',
        );

        if($addExtendedHeader){
            /* Current time in milliseconds since midnight, January 1, 1970 UTC. */
            $current_time_in_millis = $this->getTimeStamp();

            /* Creating the Hash using SHA-512 algorithm */
            $string_to_hash = $this->defaultCustomerId .$current_time_in_millis . $this->defaultCustomerApiKey;
            $hashValue = hash("sha512", $string_to_hash);
            $timestamp_header = number_format($current_time_in_millis, 0, '', '' );
            $header=array_merge($header,array("Customer-Key"=>$this->defaultCustomerId,
                "Timestamp"=>$timestamp_header, "Authorization"=>$hashValue));
        }
        return $header;
    }

    public function getTimeStamp()
    {
        $url = MiniorangeOAuthServerConstants::BASE_URL.'/moas/rest/mobile/get-timestamp';
        $fields = array();
        $currentTimeInMillis = $this->callService($url, $fields);
        if (empty($currentTimeInMillis)) {
            $currentTimeInMillis = round(microtime(true) * 1000);
            $currentTimeInMillis = number_format($currentTimeInMillis, 0, '', '');
        }
        return $currentTimeInMillis;
    }
}
